@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -30,7 +30,7 @@ class RegisteredBlockAdmin(RegisteredItemAdmin):
     fieldsets = (
         ('', {'fields': ('name', 'module', 'class_name', )}),
         (_('Status'),
-            {'fields': ('placed_at', 'active', 'order', 'config')}
+            {'fields': ('placed_at', 'active', 'shown_in_urls', 'hidden_in_urls', 'order', 'config')}
         ))
 
     def has_add_permission(self, request):
